#!/usr/bin/env python
# -*- coding: utf-8 -*-
import click

from pyparallel import Downloader


@click.command()
@click.option('--url', required=True, help='Download URL')
@click.option('--conns', required=True, help='Number of parts')
@click.option('--filename', required=True, help='Output filename')
def cli(url, conns, filename):
    downloader = Downloader(
        url=url,
        conns=int(conns),
        filename=filename
    )
    downloader.start()



# -*- coding: utf-8 -*-
import os.path

class TestDownloader:

    def test_downloader(self, downloder):
        downloder.start()

        # Check if the file exits
        assert os.path.exists('chart.jpg') is True



# -*- coding: utf-8 -*-
import sys
import os

import pytest

testpath = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, testpath + '/../')

from pyparallel import Downloader # noqa


@pytest.fixture
def downloder():
    return Downloader(
        url='https://upload.wikimedia.org/wikipedia/commons/f/ff/Pizigani_1367_Chart_10MB.jpg',  # noqa
        conns=10,
        filename='chart.jpg'
    )



# -*- coding: utf-8 -*-

"""Top-level package for PyParallel."""

__author__ = """Joe Paul"""
__email__ = 'joeirimpan@gmail.com'
__version__ = '0.2.1'


from .pyparallel import Downloader  # noqa
from .cli import cli  # noqa



# -*- coding: utf-8 -*-
"""
    pyparallel.py
    :copyright: (c) 2019 by Joe Paul.
    :license: see LICENSE for details.
"""
import os
import time
import multiprocessing as mp

import requests
from requests.exceptions import HTTPError


session = requests.Session()
session.stream = True


class Part:
    __slots__ = ['url', 'offset', 'size']

    def __init__(self, url, size, offset):
        self.url = url
        self.offset = offset
        self.size = size


def producer(part, q):
    """Download given part and push to consumer queue

    :param part: Part instance
    :param q: Consumer queue
    """
    response = session.get(
        url=part.url,
        headers={
            'Range': 'bytes=%d-%d' % (
                part.offset,
                part.offset + (part.size-1)
            )
        },
    )

    try:
        response.raise_for_status()
    except HTTPError:
        raise

    # Push data into queue
    q.put((part.offset, response.content))


def consumer(q, name):
    """Listen to consumer queue and write file using offset

    :param q: Consumer queue
    :param name: file name
    """
    f = open(name, 'w+b')

    while 1:
        m = q.get()
        if m == 'kill':
            break

        # Unpack tuple
        offset, content = m

        f.seek(offset)
        f.write(content)
        f.flush()

    f.close()


class Downloader:

    def __init__(self, url, conns, filename):
        self.url = url
        self.conns = conns
        self.filename = filename

    def start(self):
        """Analyze the url content size and create parts for later downloading
        """
        response = requests.head(self.url)
        try:
            response.raise_for_status()
        except HTTPError:
            raise

        content_length = int(response.headers.get('Content-Length'))
        if os.path.exists(self.filename):
            os.remove(self.filename)

        # Init a process manager
        manager = mp.Manager()
        q = manager.Queue()
        pool = mp.Pool(mp.cpu_count() + 2)

        # File writer consumer which listens to queue
        pool.apply_async(consumer, (q, self.filename))

        size = content_length / self.conns
        offset = 0
        jobs = []
        for index in range(self.conns):
            offset = index * size
            if index == self.conns - 1:
                size = content_length - size * index

            part = Part(**{
                'url': self.url,
                'size': size,
                'offset': offset,
            })

            job = pool.apply_async(producer, (part, q))
            jobs.append(job)

        # Collect all job results
        for job in jobs:
            job.get()

        # Kill the consumer
        q.put('kill')
        pool.close()
        pool.join()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""
import warnings
from setuptools import setup, find_packages, Command
from setuptools.command.install import install as _install

with open('README.md') as readme_file:
    readme = readme_file.read()

requirements = [
    'requests',
    'Click'
]

setup_requirements = [
    'pytest-runner'
]

test_requirements = [
    'pytest'
]

class install(_install):
    def run(self):
        _install.do_egg_install(self)

class FakeBdist(Command):
    """Fake bdist wheel class for ignoring bdist_wheel build
    """

    user_options = [(
        "dist-dir=",
        "d",
        "directory to put final built distributions in [default: dist]"
    ), (
        "python-tag=",
        None,
        "Python tag (cp32|cp33|cpNN) for abi3 wheel tag (default:false)"
    )]

    def initialize_options(self):
        self.dist_dir = None
        self.python_tag = None

    def finalize_options(self):
        pass

    def run(self):
        warnings.warn(
            "{name}{version} does not support building wheels".format(
                name=about["__title__"],
                version=about["__version__"]
            )
        )

setup(
    name='split-downloader',
    version='0.2.4',
    description="Parallel downloader",
    long_description=readme,
    author="Joe Paul",
    author_email='joeirimpan@gmail.com',
    url='https://github.com/joeirimpan/pyparallel',
    packages=find_packages(include=['pyparallel']),
    include_package_data=True,
    install_requires=requirements,
    entry_points='''
        [console_scripts]
        download=pyparallel:cli
    ''',
    license="MIT license",
    zip_safe=False,
    keywords='pyparallel',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    long_description_content_type='text/markdown',
    setup_requires=setup_requirements,
    cmdclass={"install": install, "bdist_wheel": FakeBdist}
)
